#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['nbcollate']

package_data = \
{'': ['*']}

install_requires = \
['nbformat~=4.4.0']

setup(name='nbcollate',
      version='0.3.1',
      description='Collate Jupyter classroom assignment and submission notebooks',
      author='Oliver Steele',
      author_email='steele@osteele.com',
      url='https://github.com/osteele/nbcollate',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.5',
     )
